==> cook_2.5.orig.tar.gz: cook-2.5.orig/cook/builtin/options.c <==
/*
 *	cook - file construction tool
 *	Copyright (C) 1996, 1997, 1998 Peter Miller;
 *	All rights reserved.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 * MANIFEST: functions to implement the builtin options function
 */

#include <builtin/options.h>
#include <error_intl.h>
#include <expr/position.h>
#include <option.h>
#include <str_list.h>
#include <trace.h>


typedef struct table_ty table_ty;
struct table_ty
{
	int	option;
	char	*set;
	char	*unset;
};

static table_ty table[] =
{
	{ OPTION_ACTION,	"-action",	"-noaction",	},
	{ OPTION_CASCADE,	"-cascade",	"-nocascade",	},
	{ OPTION_ERROK,		"-errok",	"-noerrok",	},
	{ OPTION_FINGERPRINT,	"-fingerprint",	"-nofingerprint", },
	{ OPTION_FORCE,		"-force",	"-noforce",	},
	{ OPTION_METER,		"-meter",	"-nometer",	},
	{ OPTION_PERSEVERE,	"-continue",	"-nocontinue",	},
	{ OPTION_PRECIOUS,	"-precious",	"-noprecious",	},
	{ OPTION_REASON,	"-reason",	"-noreason",	},
	{ OPTION_SHALLOW,	"-shallow",	"-noshallow",	},
	{ OPTION_SILENT,	"-silent",	"-nosilent",	},
	{ OPTION_STAR,		"-star",	"-nostar",	},
	{ OPTION_STRIP_DOT,	"-stripdot",	"-nostripdot",	},
	{ OPTION_TERMINAL,	"-terminal",	"-noterminal",	},
	{ OPTION_TOUCH,		"-touch",	"-notouch",	},
	{ OPTION_UPDATE,	"-update",	"-noupdate",	},
};


/*
 * NAME
 *	builtin_options - describe operating system
 *
 * SYNOPSIS
 *	int builtin_options(string_list_ty *result, string_list_ty *args);
 *
 * DESCRIPTION
 *	Operating_system is a built-in function of cook, described as
 *	follows: This function must have zero arguments.
 *
 * RETURNS
 *	The resulting wordlist contains the values of various cook
 *	options, suitable for use on a recursive cook command line.  All
 *	command line options are emitted, to override any environment
 *	variable or default settings.  The options emitted are the
 *	current union of the command line options given to cook, the
 *	settings in the COOK environment variable, and any current
 *	``set'' clause actions.
 *
 * CAVEAT
 *	The returned result is in dynamic memory.
 *	It is the responsibility of the caller to dispose of
 *	the result when it is finished, with a string_list_destructor() call.
 */

int
builtin_options(result, args, pp, ocp)
	string_list_ty	*result;
	const string_list_ty *args;
	const expr_position_ty *pp;
	const struct opcode_context_ty *ocp;
{
	table_ty	*tp;
	string_ty	*s;
	size_t		j;

	trace(("options\n"));
	if (args->nstrings != 1)
	{
		sub_context_ty	*scp;

		scp = sub_context_new();
		sub_var_set(scp, "Name", "%S", args->string[0]);
		error_with_position
		(
			pp,
			scp,
			i18n("$name: requires no arguments")
		);
		sub_context_delete(scp);
		return -1;
	}

	/*
	 * do the simple options
	 */
	for (tp = table; tp < ENDOF(table); ++tp)
	{
		s = str_from_c(option_test(tp->option) ? tp->set : tp->unset);
		string_list_append(result, s);
		str_free(s);
	}

	/*
	 * the include options next
	 */
	for (j = 0; j < option.o_search_path.nstrings; ++j)
	{
		s = str_format("-I%S", option.o_search_path.string[j]);
		string_list_append(result, s);
		str_free(s);
	}

	/*
	 * any recursive cook must have no logging, because it will
	 * inherit its log file from the parent.
	 */
	s = str_from_c("-nolog");
	string_list_append(result, s);
	str_free(s);
	return 0;
}

==> cook_2.10.orig.tar.gz: cook-2.10/cook/builtin/options.c <==
/*
 *	cook - file construction tool
 *	Copyright (C) 1996, 1997, 1998, 1999 Peter Miller;
 *	All rights reserved.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 * MANIFEST: functions to implement the builtin options function
 */

#include <builtin/options.h>
#include <error_intl.h>
#include <expr/position.h>
#include <option.h>
#include <str_list.h>
#include <trace.h>


typedef struct table_ty table_ty;
struct table_ty
{
	int	option;
	char	*set;
	char	*unset;
};

static table_ty table[] =
{
	{ OPTION_ACTION,	"-action",	"-noaction",	},
	{ OPTION_CASCADE,	"-cascade",	"-nocascade",	},
	{ OPTION_ERROK,		"-errok",	"-noerrok",	},
	{ OPTION_FINGERPRINT,	"-fingerprint",	"-nofingerprint", },
	{ OPTION_FORCE,		"-force",	"-noforce",	},
	{ OPTION_METER,		"-meter",	"-nometer",	},
	{ OPTION_PERSEVERE,	"-continue",	"-nocontinue",	},
	{ OPTION_PRECIOUS,	"-precious",	"-noprecious",	},
	{ OPTION_REASON,	"-reason",	"-noreason",	},
	{ OPTION_SHALLOW,	"-shallow",	"-noshallow",	},
	{ OPTION_SILENT,	"-silent",	"-nosilent",	},
	{ OPTION_STAR,		"-star",	"-nostar",	},
	{ OPTION_STRIP_DOT,	"-stripdot",	"-nostripdot",	},
	{ OPTION_TERMINAL,	"-terminal",	"-noterminal",	},
	{ OPTION_TOUCH,		"-touch",	"-notouch",	},
	{ OPTION_UPDATE,	"-timeadjust",	"-notimeadjust", },
};


/*
 * NAME
 *	builtin_options - describe operating system
 *
 * SYNOPSIS
 *	int builtin_options(string_list_ty *result, string_list_ty *args);
 *
 * DESCRIPTION
 *	Operating_system is a built-in function of cook, described as
 *	follows: This function must have zero arguments.
 *
 * RETURNS
 *	The resulting wordlist contains the values of various cook
 *	options, suitable for use on a recursive cook command line.  All
 *	command line options are emitted, to override any environment
 *	variable or default settings.  The options emitted are the
 *	current union of the command line options given to cook, the
 *	settings in the COOK environment variable, and any current
 *	``set'' clause actions.
 *
 * CAVEAT
 *	The returned result is in dynamic memory.
 *	It is the responsibility of the caller to dispose of
 *	the result when it is finished, with a string_list_destructor() call.
 */

static int interpret _((string_list_ty *, const string_list_ty *,
	const expr_position_ty *, const struct opcode_context_ty *));

static int
interpret(result, args, pp, ocp)
	string_list_ty	*result;
	const string_list_ty *args;
	const expr_position_ty *pp;
	const struct opcode_context_ty *ocp;
{
	table_ty	*tp;
	string_ty	*s;
	size_t		j;

	trace(("options\n"));
	if (args->nstrings != 1)
	{
		sub_context_ty	*scp;

		scp = sub_context_new();
		sub_var_set(scp, "Name", "%S", args->string[0]);
		error_with_position
		(
			pp,
			scp,
			i18n("$name: requires no arguments")
		);
		sub_context_delete(scp);
		return -1;
	}

	/*
	 * do the simple options
	 */
	for (tp = table; tp < ENDOF(table); ++tp)
	{
		s = str_from_c(option_test(tp->option) ? tp->set : tp->unset);
		string_list_append(result, s);
		str_free(s);
	}

	/*
	 * the include options next
	 */
	for (j = 0; j < option.o_search_path.nstrings; ++j)
	{
		s = str_format("-I%S", option.o_search_path.string[j]);
		string_list_append(result, s);
		str_free(s);
	}

	/*
	 * any recursive cook must have no logging, because it will
	 * inherit its log file from the parent.
	 */
	s = str_from_c("-nolog");
	string_list_append(result, s);
	str_free(s);
	return 0;
}


builtin_ty builtin_options =
{
	"options",
	interpret,
	interpret, /* script */
};
